import { world, system, BlockPermutation } from '@minecraft/server'

world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.blockComponentRegistry.registerCustomComponent('pvz:family_locator', {
		onTick(e) {
			const { block } = e;
			const {x,y,z} = block.location;
			const unplantable = block.permutation.withState("pvz:plantable", false);
			const plantable = block.permutation.withState("pvz:plantable", true);
			block.setPermutation(plantable);
			const systemRun = system.run(() => {
				const entities = block.dimension.getEntities({
					location:{ x: block.location.x +0.5, y: block.location.y +0.5, z: block.location.z +0.5 },
					maxDistance: 0.75,
					families: ["plant"],
				});
				for(const entity of entities) {
					if(entity.matches({ families: ["plant"]}) && block.permutation.getState("pvz:absorbable") === true) {
						block.setPermutation(unplantable);
						if(entity.matches({ families: ["pea"] })) {
							const absorb = block.permutation.withState("pvz:family", 'pea');
							block.setPermutation(absorb);
						}
						else if(entity.matches({ families: ["light"] })) {
							const absorb = block.permutation.withState("pvz:family", 'light');
							block.setPermutation(absorb);
						}
						else if(entity.matches({ families: ["winter"] })) {
							const absorb = block.permutation.withState("pvz:family", 'winter');
							block.setPermutation(absorb);
						}
						else if(entity.matches({ families: ["contain"] })) {
							const absorb = block.permutation.withState("pvz:family", 'contain');
							block.setPermutation(absorb);
						}
						else if(entity.matches({ families: ["aqua"] })) {
							const absorb = block.permutation.withState("pvz:family", 'aqua');
							block.setPermutation(absorb);
						}
						else if(entity.matches({ families: ["pepper"] })) {
							const absorb = block.permutation.withState("pvz:family", 'pepper');
							block.setPermutation(absorb);
						}
						else if(entity.matches({ families: ["fila"] })) {
							const absorb = block.permutation.withState("pvz:family", 'fila');
							block.setPermutation(absorb);
						}
						else if(entity.matches({ families: ["relic"] })) {
							const absorb = block.permutation.withState("pvz:family", 'relic');
							block.setPermutation(absorb);
						}
					}
					else if(entity.matches({ families: ["plant"]}) && block.permutation.getState("pvz:absorbable") === false) {
						block.setPermutation(unplantable);
					}
				};
			});
		},
	});
});

world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.blockComponentRegistry.registerCustomComponent('pvz:class_hability', {
		onTick(e) {
			const { block } = e;
			const {x,y,z} = block.location;
			const hollow = [
				"minecraft:air"
			];
			const clean = [
				"pvz:pea_sprout",
				"pvz:rainbow_stella"
			];
			
			if((block.permutation.getState("pvz:family") === 'none')) {
				if((clean.includes(block.below(1)?.typeId))) {
					block.below().setType('minecraft:air');
				}
			}
			else if((block.permutation.getState("pvz:family") === 'pea') && (hollow.includes(block.below(1)?.typeId))) {
				block.below().setType('pvz:pea_sprout');
				block.dimension.playSound("dig.fungus");
			}
			else if((block.permutation.getState("pvz:family") === 'light') && (hollow.includes(block.below(1)?.typeId))) {
				block.below().setType('pvz:rainbow_stella');
				block.dimension.playSound("chime.amethyst_block");
			}
			else if((block.permutation.getState("pvz:family") === 'winter')) {
				block.dimension.spawnParticle('pvz:freeze_area', { x: block.location.x + 0.5, y: block.location.y + 0.1, z: block.location.z + 0.5})
				if((clean.includes(block.below(1)?.typeId))) {
					block.below().setType('minecraft:air');
				}
				const systemRun = system.run(() => {
					const entities = block.dimension.getEntities({
						location:{ x: block.location.x +0.5, y: block.location.y +0.5, z: block.location.z +0.5 },
						maxDistance: 5.0,
						families: ["monster"],
					});
					for(const entity of entities) {
						if(entity.matches({ families: ["monster"] })) {
							entity.runCommand('effect @s slowness 10 0 false');
						}
					};
				});
			}
			else if((block.permutation.getState("pvz:family") === 'contain')) {
				if((clean.includes(block.below(1)?.typeId))) {
					block.below().setType('minecraft:air');
				}
			}
			else if((block.permutation.getState("pvz:family") === 'aqua')) {
				if((clean.includes(block.below(1)?.typeId))) {
					block.below().setType('minecraft:air');
				}
			}
			else if((block.permutation.getState("pvz:family") === 'pepper')) {
				block.dimension.spawnParticle('pvz:fire_area', { x: block.location.x + 0.5, y: block.location.y + 0.1, z: block.location.z + 0.5})
				if((clean.includes(block.below(1)?.typeId))) {
					block.below().setType('minecraft:air');
				}
				const systemRun = system.run(() => {
					const entities = block.dimension.getEntities({
						location:{ x: block.location.x +0.5, y: block.location.y +0.5, z: block.location.z +0.5 },
						maxDistance: 5.0,
						families: ["monster"],
					});
					for(const entity of entities) {
						if(entity.matches({ families: ["monster"] })) {
							entity.setOnFire(1);
						}
					};
				});
			}
			else if((block.permutation.getState("pvz:family") === 'fila')) {
				if((clean.includes(block.below(1)?.typeId))) {
					block.below().setType('minecraft:air');
				}
			}
			else if((block.permutation.getState("pvz:family") === 'relic')) {
				if((clean.includes(block.below(1)?.typeId))) {
					block.below().setType('minecraft:air');
				}
			}
		},
	});
});

world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.blockComponentRegistry.registerCustomComponent('pvz:secondary_hability', {
		onRandomTick(e) {
			const { block } = e;
			const {x,y,z} = block.location;
			const hollow = [
				"minecraft:air"
			];
			
			system.runTimeout(() => {
				if((block.permutation.getState("pvz:family") === 'contain') && (hollow.includes(block.below(1)?.typeId))) {
					block.dimension.spawnEntity('shoot:caramel_cloud', { x: block.location.x +0.5, y: block.location.y -0.1, z: block.location.z +0.5 });
				}
				else if((block.permutation.getState("pvz:family") === 'aqua') && (hollow.includes(block.below(1)?.typeId))) {
					block.dimension.spawnEntity('shoot:water_cloud', { x: block.location.x +0.5, y: block.location.y -0.1, z: block.location.z +0.5 });
				}
				else if((block.permutation.getState("pvz:family") === 'fila') && (hollow.includes(block.below(1)?.typeId))) {
					block.dimension.spawnEntity('shoot:fila_cloud', { x: block.location.x +0.5, y: block.location.y -0.1, z: block.location.z +0.5 });
				}
				else if((block.permutation.getState("pvz:family") === 'relic') && (hollow.includes(block.below(1)?.typeId))) {
					block.dimension.spawnEntity('anim:gold', { x: block.location.x +0.5, y: block.location.y -0.1, z: block.location.z +0.5 });
				}
			}, 500);
		},
	});
});

world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.blockComponentRegistry.registerCustomComponent('pvz:damage_zombie', {
		onTick(e) {
			const { block } = e;
			const {x,y,z} = block.location;
			const systemRun = system.run(() => {
				const entities = block.dimension.getEntities({
					location:{ x: block.location.x +0.5, y: block.location.y +0.5, z: block.location.z +0.5 },
					maxDistance: 1.0,
					families: ["monster"],
				});
				for(const entity of entities) {
					if(entity.matches({ families: ["monster"] })) {
						entity.runCommand('effect @s wither 5 1 false');
					}
				};
			});
		},
	});
});

world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.blockComponentRegistry.registerCustomComponent('pvz:healing_plants', {
		onTick(e) {
			const { block } = e;
			const {x,y,z} = block.location;
			const systemRun = system.run(() => {
				const entities = block.dimension.getEntities({
					location:{ x: block.location.x +0.5, y: block.location.y +0.5, z: block.location.z +0.5 },
					maxDistance: 4.0,
					families: ["plant"],
				});
				for(const entity of entities) {
					if(entity.matches({ families: ["plant"] })) {
						entity.runCommand('effect @s regeneration 10 0 false');
					}
				};
			});
		},
	});
});

world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.blockComponentRegistry.registerCustomComponent('pvz:rainbow_road', {
		onTick(e) {
			const { block } = e;
			const {x,y,z} = block.location;
			const hollow = [
				"minecraft:air"
			];
			const filled = [
				"pvz:rainbow_stella"
			];
			system.runTimeout(() => {
				const stop = block.permutation.withState("pvz:placed", false);
				block.setPermutation(stop);
			}, 5);
			if((hollow.includes(block.below(1)?.typeId)) && (filled.includes(block.below(1)?.typeId) === false)) {
				block.below().setType('pvz:rainbow_stella');
			}
			
			system.runTimeout(() => {
				const ghost = block.permutation.withState("pvz:clean", false);
				block.setPermutation(ghost);
			}, 5);
			system.runTimeout(() => {
				const fast = block.permutation.withState("pvz:placed", true);
				block.setPermutation(fast);
			}, 200);
		},
	});
});

world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.blockComponentRegistry.registerCustomComponent('pvz:magic_bean', {
		onTick(e) {
			const { block } = e;
			const {x,y,z} = block.location;
			const hollow = [
				"minecraft:air"
			];
			const filled = [
				"pvz:pea_sprout"
			];
			system.runTimeout(() => {
				const stop = block.permutation.withState("pvz:placed", false);
				block.setPermutation(stop);
			}, 5);
			if((hollow.includes(block.below(1)?.typeId)) && (filled.includes(block.below(1)?.typeId) === false)) {
				block.below().setType('pvz:pea_sprout');
			}
			system.runTimeout(() => {
				const fast = block.permutation.withState("pvz:placed", true);
				block.setPermutation(fast);
			}, 600);
		},
	});
});

world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.blockComponentRegistry.registerCustomComponent('pvz:drop_peas', {
		onRandomTick(e) {
			const { block } = e;
			const {x,y,z} = block.location;
			
			system.runTimeout(() => {
				if(block.permutation.getState("pvz:pea") === true) {
					block.dimension.spawnEntity('anim:pea_vines', { x: block.location.x +0.5, y: block.location.y +0.5, z: block.location.z +0.5 });
				}
			}, 500);
		}
	});
});

world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.blockComponentRegistry.registerCustomComponent('pvz:clear_puddle', {
		onPlace: e => {
			const { block } = e;
			const {x,y,z} = block.location;
			const despawn = 'minecraft:air';
			system.runTimeout(() => {
				block.setPermutation(BlockPermutation.resolve(`${despawn}`))
			}, 1200);
		}
	});
});

world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.blockComponentRegistry.registerCustomComponent('pvz:dry_out', {
		onTick(e) {
			const { block } = e;
			const {x,y,z} = block.location;
			const despawn = 'minecraft:air';
			const hollow = [
				"minecraft:air",
				"pvz:water_puddle",
				"pvz:caramel_puddle",
				"pvz:butter_puddle",
				"pvz:pool_slot",
				"pvz:blue_pool_slot",
				"minecraft:water",
				"minecraft:flowing_water",
				"minecraft:lava",
				"minecraft:flowing_lava",
				"minecraft:tall_grass",
				"minecraft:short_grass",
				"minecraft:fern",
				"minecraft:large_fern",
				"minecraft:dandelion",
				"minecraft:poppy"
			];
			if((hollow.includes(block.below(1)?.typeId))) {
				block.setPermutation(BlockPermutation.resolve(`${despawn}`))
			}
		},
	});
});

world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.blockComponentRegistry.registerCustomComponent('pvz:retain', {
		onTick(e) {
			const { block } = e;
			const {x,y,z} = block.location;
			const systemRun = system.run(() => {
				const entities = block.dimension.getEntities({
					location:{ x: block.location.x +0.5, y: block.location.y +0.5, z: block.location.z +0.5 },
					maxDistance: 1.0,
					families: ["monster"],
				});
				for(const entity of entities) {
					if(entity.matches({ families: ["monster"] })) {
						entity.runCommand('effect @s slowness 10 1 false');
						entity.runCommand('effect @s weakness 10 0 false');
						entity.runCommand('effect @s mining_fatigue 10 1 false');
					}
				};
			});
		},
	});
});

world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.blockComponentRegistry.registerCustomComponent('pvz:water_boost', {
		onTick(e) {
			const { block } = e;
			const {x,y,z} = block.location;
			const systemRun = system.run(() => {
				const entities = block.dimension.getEntities({
					location:{ x: block.location.x +0.5, y: block.location.y +0.5, z: block.location.z +0.5 },
					maxDistance: 1.0,
					families: ["plant"],
				});
				for(const entity of entities) {
					if(entity.matches({ families: ["plant"] })) {
						entity.runCommand('effect @s haste 10 1 false');
					}
				};
			});
		},
	});
});

world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.blockComponentRegistry.registerCustomComponent('pvz:butter_mob', {
		onTick(e) {
			const { block } = e;
			const {x,y,z} = block.location;
			const despawn = 'minecraft:air';
			const systemRun = system.run(() => {
				const entities = block.dimension.getEntities({
					location:{ x: block.location.x +0.5, y: block.location.y +0.5, z: block.location.z +0.5 },
					maxDistance: 1.0,
					families: ["monster"],
				});
				for(const entity of entities) {
					if(entity.matches({ families: ["monster"] })) {
						entity.runCommand('execute unless entity @e[type=pvz:butter, c=1, x=~, y=~+1, z=~, dy=2] run summon pvz:butter ~ ~ ~');
						entity.runCommand('ride @s start_riding @e[type=pvz:butter, c=1, r=1.5] teleport_ride');
						block.setPermutation(BlockPermutation.resolve(`${despawn}`))
					}
				};
			});
		},
	});
});