import { world, system, BlockPermutation, ItemStack, GameMode } from '@minecraft/server'

world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.blockComponentRegistry.registerCustomComponent('egypt:common_loot', {
        onPlayerInteract(e) {
            const { player, block } = e;
            const {x,y,z} = block.location;
			const equipment = player.getComponent('equippable');
			const selectedItem = equipment.getEquipment('Mainhand');
			const close = block.permutation.withState("pvz:closed", true);
			const open = block.permutation.withState("pvz:closed", false);
			const hasClaimed = player.getDynamicProperty(`opened: ${x} ${y} ${z}`) ?? false;
			
			if (hasClaimed && selectedItem && selectedItem.typeId === 'pvz:egypt_key' && block.permutation.getState("pvz:closed") === true) {
				player.playSound("vault.insert_item_fail");
				e.cancel = true;
				return;
			};
			
			if(selectedItem && selectedItem.typeId === 'pvz:egypt_key' && block.permutation.getState("pvz:closed") === true && block.permutation.getState("pvz:category") === 'common') {
				block.setPermutation(open);
				player.playSound("vault.open_shutter");
				player.setDynamicProperty(`opened: ${x} ${y} ${z}`, true);
				
				if (!player.matches({ gameMode: GameMode.creative}) && selectedItem.amount > 1) {
					selectedItem.amount -= 1;
					equipment.setEquipment('Mainhand', selectedItem);
				}
				else if (!player.matches({ gameMode: GameMode.creative}) && selectedItem.amount === 1) {
					equipment.setEquipment('Mainhand', new ItemStack('minecraft:air', 1));
				}
				
				system.runTimeout(() => {
					block.dimension.spawnEntity('anim:egypt_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
				}, 20);
				system.runTimeout(() => {
					block.dimension.spawnEntity('anim:egypt_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
				}, 40);
				system.runTimeout(() => {
					block.dimension.spawnEntity('anim:egypt_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
				}, 60);
				system.runTimeout(() => {
					block.dimension.spawnEntity('anim:egypt_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
				}, 80);
				system.runTimeout(() => {
					player.playSound("vault.deactivate");
					block.setPermutation(close);
				}, 100);
			}
		}
	});
});

world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.blockComponentRegistry.registerCustomComponent('egypt:special_loot', {
        onPlayerInteract(e) {
            const { player, block } = e;
            const {x,y,z} = block.location;
			const equipment = player.getComponent('equippable');
			const selectedItem = equipment.getEquipment('Mainhand');
			const close = block.permutation.withState("pvz:closed", true);
			const open = block.permutation.withState("pvz:closed", false);
			const hasClaimed = player.getDynamicProperty(`opened: ${x} ${y} ${z}`) ?? false;
			
			if (hasClaimed && selectedItem && selectedItem.typeId === 'pvz:egypt_key' && block.permutation.getState("pvz:closed") === true) {
				player.playSound("vault.insert_item_fail");
				e.cancel = true;
				return;
			};
			if(selectedItem && selectedItem.typeId === 'pvz:timeless_key' && block.permutation.getState("pvz:closed") === true) {
				block.setPermutation(open);
				player.playSound("vault.open_shutter");
				
				if (!player.matches({ gameMode: GameMode.creative}) && selectedItem.amount > 1) {
					selectedItem.amount -= 1;
					equipment.setEquipment('Mainhand', selectedItem);
				}
				else if (!player.matches({ gameMode: GameMode.creative}) && selectedItem.amount === 1) {
					equipment.setEquipment('Mainhand', new ItemStack('minecraft:air', 1));
				}
				
				if(block.permutation.getState("pvz:category") === 'blockbuster') {
					system.runTimeout(() => {
						block.dimension.spawnEntity('anim:blockbuster_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
					}, 20);
					system.runTimeout(() => {
						block.dimension.spawnEntity('anim:blockbuster_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
					}, 40);
					system.runTimeout(() => {
						block.dimension.spawnEntity('anim:blockbuster_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
					}, 60);
					system.runTimeout(() => {
						block.dimension.spawnEntity('anim:blockbuster_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
						block.dimension.spawnEntity('anim:blockbuster_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
						block.dimension.spawnEntity('anim:blockbuster_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
					}, 100);
					system.runTimeout(() => {
						player.playSound("vault.deactivate");
						block.setPermutation(close);
					}, 120);
				}
				else if(block.permutation.getState("pvz:category") === 'gloomshroom') {
					system.runTimeout(() => {
						block.dimension.spawnEntity('anim:gloomshroom_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
					}, 20);
					system.runTimeout(() => {
						block.dimension.spawnEntity('anim:gloomshroom_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
					}, 40);
					system.runTimeout(() => {
						block.dimension.spawnEntity('anim:gloomshroom_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
					}, 60);
					system.runTimeout(() => {
						block.dimension.spawnEntity('anim:gloomshroom_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
						block.dimension.spawnEntity('anim:gloomshroom_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
						block.dimension.spawnEntity('anim:gloomshroom_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
					}, 100);
					system.runTimeout(() => {
						player.playSound("vault.deactivate");
						block.setPermutation(close);
					}, 120);
				}
				else if(block.permutation.getState("pvz:category") === 'snowpea') {
					system.runTimeout(() => {
						block.dimension.spawnEntity('anim:snowpea_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
					}, 20);
					system.runTimeout(() => {
						block.dimension.spawnEntity('anim:snowpea_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
					}, 40);
					system.runTimeout(() => {
						block.dimension.spawnEntity('anim:snowpea_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
					}, 60);
					system.runTimeout(() => {
						block.dimension.spawnEntity('anim:snowpea_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
						block.dimension.spawnEntity('anim:snowpea_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
						block.dimension.spawnEntity('anim:snowpea_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
					}, 100);
					system.runTimeout(() => {
						player.playSound("vault.deactivate");
						block.setPermutation(close);
					}, 120);
				}
				else if(block.permutation.getState("pvz:category") === 'pumpkin') {
					system.runTimeout(() => {
						block.dimension.spawnEntity('anim:pumpkin_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
					}, 20);
					system.runTimeout(() => {
						block.dimension.spawnEntity('anim:pumpkin_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
					}, 40);
					system.runTimeout(() => {
						block.dimension.spawnEntity('anim:pumpkin_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
					}, 60);
					system.runTimeout(() => {
						block.dimension.spawnEntity('anim:pumpkin_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
						block.dimension.spawnEntity('anim:pumpkin_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
						block.dimension.spawnEntity('anim:pumpkin_loot', { x: block.location.x +0.5, y: block.location.y +1.1, z: block.location.z +0.5 });
					}, 100);
					system.runTimeout(() => {
						player.playSound("vault.deactivate");
						block.setPermutation(close);
					}, 120);
				}
			}
		}
	});
});