import { world, system, BlockPermutation, ItemStack, GameMode } from '@minecraft/server'

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent('pvz:spawner', {
		onRandomTick: e => {
			const { block } = e;
            const {x,y,z} = block.location;
			
			block.dimension.spawnEntity('anim:mummy', { x: block.location.x +0.5, y: block.location.y +0.75, z: block.location.z +0 });
			block.dimension.playSound("block.composter.ready", block.location);
		}
	});
});

world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.blockComponentRegistry.registerCustomComponent('pvz:gravebuster_spawner', {
		onPlayerInteract(e) {
			const { player, block } = e;
			const {x,y,z} = block.location;
			const equipment = player.getComponent('equippable');
			const selectedItem = equipment.getEquipment('Mainhand');
			const excludeSeeds = [
				"pvz:sunflower_seed",
				"pvz:peashooter_seed",
				"pvz:wallnut_seed",
				"pvz:potatomine_seed",
				"pvz:cabbage_seed",
				"pvz:bloomerang_seed",
				"pvz:iceberg_seed",
				"bonk_choy_seed",
				"pvz:repeater_seed",
				"pvz:twinflower_seed",
				"pvz:pumpkin_seed",
				"pvz:gloomshroom_seed",
				"pvz:snowpea_seed",
				"pvz:marigold_seed",
				"pvz:megagatling_seed",
				"pvz:birth_sunflower_seed",
				"pvz:doomshroom_seed",
				"pvz:fireboal_seed",
				"pvz:brick_pot",
				"pvz:kernel_seed",
				"pvz:cornie_seed"
			]
			
			if(selectedItem && selectedItem.typeId === 'pvz:gravebuster_seed') {
			
				block.dimension.spawnEntity('pvz:gravebuster', { x: block.location.x +0.5, y: block.location.y +0, z: block.location.z +0.5 });
				
				if (!player.matches({ gameMode: GameMode.creative}) && selectedItem.amount > 1) {
					selectedItem.amount -= 1;
					equipment.setEquipment('Mainhand', selectedItem);
				}
				else if (!player.matches({ gameMode: GameMode.creative}) && selectedItem.amount === 1) {
					equipment.setEquipment('Mainhand', new ItemStack('minecraft:air', 1));
				}
			}
			else if(selectedItem && selectedItem.typeId === 'pvz:blockbuster_seed') {
			
				block.dimension.spawnEntity('pvz:blockbuster<minecraft:gravestoned>', { x: block.location.x +0.5, y: block.location.y +0, z: block.location.z +0.5 });
				
				if (!player.matches({ gameMode: GameMode.creative}) && selectedItem.amount > 1) {
					selectedItem.amount -= 1;
					equipment.setEquipment('Mainhand', selectedItem);
				}
				else if (!player.matches({ gameMode: GameMode.creative}) && selectedItem.amount === 1) {
					equipment.setEquipment('Mainhand', new ItemStack('minecraft:air', 1));
				}
			}
			else if(excludeSeeds.includes(selectedItem.typeId)) {
				
				player.runCommand('titleraw @p actionbar {"rawtext":[{"translate":"action.plant.deny"}]}');
			}
		}
	});
});

world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.blockComponentRegistry.registerCustomComponent('pvz:gravebuster_checker', {
		onTick(e) {
			const { block } = e;
			const {x,y,z} = block.location;
			const hide = block.permutation.withState("pvz:visibility", false);
			const unhide = block.permutation.withState("pvz:visibility", true);
			block.setPermutation(unhide);
			const systemRun = system.run(() => {
				const entities = block.dimension.getEntities({
					location:{ x: block.location.x +0.5, y: block.location.y +0.1, z: block.location.z +0.5 },
					maxDistance: 0.5
				});
				for(const entity of entities) {
					if(entity.typeId === "pvz:gravebuster") {
						block.setPermutation(hide);
					}
					else if(entity.typeId === "pvz:blockbuster") {
					}
				};
			});
		},
	});
});

world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.blockComponentRegistry.registerCustomComponent('pvz:egypt_garg_spawner', {
		onTick(e) {
			const { block } = e;
			const {x,y,z} = block.location;
			const spawn = block.permutation.withState("pvz:spawned", true);
			const systemRun = system.run(() => {
				const entities = block.dimension.getEntities({
					location:{ x: block.location.x +0.5, y: block.location.y +0.1, z: block.location.z +0.5 },
					maxDistance: 8.0
				});
				for(const entity of entities) {
					if(entity.typeId === "minecraft:player"  && block.permutation.getState("pvz:spawned") === false) {
						entity.runCommand('titleraw @p title {"rawtext":[{"translate":"grave.ready"}]}');
						block.setPermutation(spawn);
						system.runTimeout(() => {
							entity.runCommand('titleraw @p title {"rawtext":[{"translate":"grave.set"}]}');
						}, 30);
						system.runTimeout(() => {
							entity.runCommand('titleraw @p title {"rawtext":[{"translate":"grave.fight"}]}');
							block.dimension.spawnEntity('pvz:egypt_gargantuar', { x: block.location.x +0.5, y: block.location.y +0.1, z: block.location.z +0.5 });
						}, 60);
					}
				};
			});
		},
	});
});

world.beforeEvents.worldInitialize.subscribe(initEvent => {
	initEvent.blockComponentRegistry.registerCustomComponent('pvz:grave_placed', {
		onPlace: e => {
			const { block } = e;
			const {x,y,z} = block.location;
			const placed = block.permutation.withState("pvz:spawned", false);
			block.setPermutation(placed);
		}
	});
});